#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>

#define WIDTH 256
#define HEIGHT 256


void quit(Ecore_Evas *ecore_evas)
{
	ecore_main_loop_quit();
}

void mouse_down( void *clip, Evas *evas, Evas_Object *part, void *event_info)
{
	Evas_Event_Mouse_Down *event = event_info;
	if(event->button == 1)
		evas_object_clip_set(part, clip);
	else if(event->button == 3)
		evas_object_clip_unset(part);
	else if(event->button == 2) {
		Evas_Object *box = evas_object_name_find(evas, "box");
		evas_object_del(box);
	}
}

int
main( int argc, char **argv )
{
	puts("== clip test ==");
	puts("[LMB] = clip");
	puts("[RMB] = unclip");
	puts("[MMB] = delete edje box group");
	puts("You will notive that:");
	puts("* Once all three objects have been clipped the visible area goes white.");
	puts(" - (with the exception of the clipped objects)");
	puts("* Its interesting to note that you can unclip and clip any two objects freely,");
	puts(" - but the last one causes the white");
	puts("* It sometimes forgets to redraw.");
	puts("* Deleting the edje box group restores visibility");
	Ecore_Evas *ecore_evas;
	Evas *evas;

	/* Initialise our libraries */
	if( !ecore_evas_init() ) return EXIT_FAILURE;
	if( !edje_init() ) return EXIT_FAILURE;

	/* Create our Ecore_Evas */
	ecore_evas = ecore_evas_software_x11_new( NULL, 0, 0, 0, WIDTH,HEIGHT );
	ecore_evas_title_set( ecore_evas, "A Window" );
	ecore_evas_show( ecore_evas );

	ecore_evas_callback_delete_request_set(ecore_evas, quit);

	/* get a handle to our canvas */
	evas = ecore_evas_get( ecore_evas );

	/* create a black rect as the background */
	Evas_Object *back = evas_object_rectangle_add(evas);
	evas_object_color_set(back, 64, 64, 64, 255);
	evas_object_resize(back, WIDTH, HEIGHT);
	evas_object_move(back, 0,0);
	evas_object_layer_set(back, 1);
	evas_object_show(back);

	/* create the box group object */
	Evas_Object *box = edje_object_add( evas );
	edje_object_file_set( box, "box.edj", "default" );
	evas_object_move(box, 64,32);
	evas_object_resize( box, 128, 192 );
	evas_object_layer_set(box,2);
	evas_object_name_set(box, "box");
	evas_object_show( box );

	/* create three more objects to put into the box part */
	Evas_Object *part1 = edje_object_add(evas);
	edje_object_file_set( part1, "colours.edj", "green" );
	evas_object_resize(part1, 96, 60);
	evas_object_show(part1);

	Evas_Object *part2 = edje_object_add(evas);
	edje_object_file_set( part2, "colours.edj", "gold" );
	evas_object_resize(part2, 96, 60);
	evas_object_show(part2);

	Evas_Object *part3 = edje_object_add(evas);
	edje_object_file_set( part3, "colours.edj", "blue" );
	evas_object_resize(part3, 96, 60);
	evas_object_show(part3);

	/* add the objects to our box */
	edje_object_part_box_append( box, "contents", part1);	
	edje_object_part_box_append( box, "contents", part2);
	edje_object_part_box_append( box, "contents", part3);

	/* create the clipper object */
	Evas_Object *clip = evas_object_rectangle_add(evas);
	evas_object_color_set(clip, 64, 64, 255, 64);
	evas_object_resize(clip, 128, 128);
	evas_object_move(clip, 0,64);
	evas_object_layer_set(clip, 3);
	evas_object_show(clip);

	/*  add mousedown callback to our two parts so that they become clipped */
	evas_object_event_callback_add( part1, EVAS_CALLBACK_MOUSE_DOWN,
			(Evas_Object_Event_Cb)mouse_down, clip);
	evas_object_event_callback_add( part2, EVAS_CALLBACK_MOUSE_DOWN,
			(Evas_Object_Event_Cb)mouse_down, clip);
	evas_object_event_callback_add( part3, EVAS_CALLBACK_MOUSE_DOWN,
			(Evas_Object_Event_Cb)mouse_down, clip);

	/* Start the main event loop */
	ecore_main_loop_begin();

	/* delete the objects */
	evas_object_del(box);

	/* When the main event loop exits shutdown our libraries */
	edje_shutdown();
	ecore_evas_shutdown();
	return 0;
}


